@@ -32,6 +32,7 @@
 #include "gdk-pixbuf-io.h"
 #include "gdk-pixbuf-loader.h"
 #include "gdk-pixbuf-marshal.h"
+#include "gdk-pixbuf-alias.h"
 
 enum {
         SIZE_PREPARED,
@@ -49,7 +50,6 @@ static void gdk_pixbuf_loader_finalize      (GObject                *loader);
 static gpointer parent_class = NULL;
 static guint    pixbuf_loader_signals[LAST_SIGNAL] = { 0 };
 
-
 /* Internal data */
 
 #define LOADER_HEADER_SIZE 128
@@ -58,6 +58,7 @@ typedef struct
 {
         GdkPixbufAnimation *animation;
         gboolean closed;
+        gboolean holds_threadlock;
         guchar header_buf[LOADER_HEADER_SIZE];
         gint header_buf_offset;
         GdkPixbufModule *image_module;
@@ -134,7 +135,7 @@ gdk_pixbuf_loader_class_init (GdkPixbufLoaderClass *class)
                               G_SIGNAL_RUN_LAST,
                               G_STRUCT_OFFSET (GdkPixbufLoaderClass, size_prepared),
                               NULL, NULL,
-                              gdk_pixbuf_marshal_VOID__INT_INT,
+                              _gdk_pixbuf_marshal_VOID__INT_INT,
                               G_TYPE_NONE, 2, 
                               G_TYPE_INT,
                               G_TYPE_INT);
@@ -154,7 +155,7 @@ gdk_pixbuf_loader_class_init (GdkPixbufLoaderClass *class)
                               G_SIGNAL_RUN_LAST,
                               G_STRUCT_OFFSET (GdkPixbufLoaderClass, area_prepared),
                               NULL, NULL,
-                              gdk_pixbuf_marshal_VOID__VOID,
+                              _gdk_pixbuf_marshal_VOID__VOID,
                               G_TYPE_NONE, 0);
 
         /**
@@ -177,7 +178,7 @@ gdk_pixbuf_loader_class_init (GdkPixbufLoaderClass *class)
                               G_SIGNAL_RUN_LAST,
                               G_STRUCT_OFFSET (GdkPixbufLoaderClass, area_updated),
                               NULL, NULL,
-                              gdk_pixbuf_marshal_VOID__INT_INT_INT_INT,
+                              _gdk_pixbuf_marshal_VOID__INT_INT_INT_INT,
                               G_TYPE_NONE, 4,
                               G_TYPE_INT,
                               G_TYPE_INT,
@@ -199,7 +200,7 @@ gdk_pixbuf_loader_class_init (GdkPixbufLoaderClass *class)
                               G_SIGNAL_RUN_LAST,
                               G_STRUCT_OFFSET (GdkPixbufLoaderClass, closed),
                               NULL, NULL,
-                              gdk_pixbuf_marshal_VOID__VOID,
+                              _gdk_pixbuf_marshal_VOID__VOID,
                               G_TYPE_NONE, 0);
 }
 
@@ -224,8 +225,12 @@ gdk_pixbuf_loader_finalize (GObject *object)
         loader = GDK_PIXBUF_LOADER (object);
         priv = loader->priv;
 
-        if (!priv->closed)
+        if (!priv->closed) {
                 g_warning ("GdkPixbufLoader finalized without calling gdk_pixbuf_loader_close() - this is not allowed. You must explicitly end the data stream to the loader before dropping the last reference.");
+                if (priv->holds_threadlock) {
+                        _gdk_pixbuf_unlock (priv->image_module);
+                }
+        }
         if (priv->animation)
                 g_object_unref (priv->animation);
   
@@ -381,6 +386,10 @@ gdk_pixbuf_loader_load_module (GdkPixbufLoader *loader,
                         return 0;
                 }
 
+	if (!priv->holds_threadlock) {
+                priv->holds_threadlock = _gdk_pixbuf_lock (priv->image_module);
+        }
+
         priv->context = priv->image_module->begin_load (gdk_pixbuf_loader_size_func,
                                                         gdk_pixbuf_loader_prepare,
                                                         gdk_pixbuf_loader_update,
@@ -555,6 +564,7 @@ gdk_pixbuf_loader_new_with_type (const char *image_type,
         if (tmp != NULL)
                 {
                         g_propagate_error (error, tmp);
+                        gdk_pixbuf_loader_close (retval, NULL);
                         g_object_unref (retval);
                         return NULL;
                 }
@@ -603,17 +613,18 @@ gdk_pixbuf_loader_new_with_mime_type (const char *mime_type,
                                 image_type = info->name;
                                 break;
                         }
-        }        
+        }
 
         g_slist_free (formats);
 
         retval = g_object_new (GDK_TYPE_PIXBUF_LOADER, NULL);
 
         tmp = NULL;
-        gdk_pixbuf_loader_load_module(retval, image_type, &tmp);
+        gdk_pixbuf_loader_load_module (retval, image_type, &tmp);
         if (tmp != NULL)
                 {
                         g_propagate_error (error, tmp);
+                        gdk_pixbuf_loader_close (retval, NULL);
                         g_object_unref (retval);
                         return NULL;
                 }
@@ -735,6 +746,10 @@ gdk_pixbuf_loader_close (GdkPixbufLoader *loader,
                 }
   
         priv->closed = TRUE;
+	if (priv->image_module && priv->holds_threadlock) {
+                _gdk_pixbuf_unlock (priv->image_module);
+                priv->holds_threadlock = FALSE;
+        }
 
         if (priv->needs_scale) 
                 {
@@ -792,5 +807,6 @@ gdk_pixbuf_loader_get_format (GdkPixbufLoader *loader)
 }
 
 
-
+#define __GDK_PIXBUF_LOADER_C__
+#include "gdk-pixbuf-aliasdef.c"
 
