@@ -33,6 +33,8 @@
 #include "gdk-pixbuf-private.h"
 #include "gdk-pixbuf-io.h"
 
+#define DUMPBIH 0
+
 
 
 #if 0
@@ -217,7 +219,19 @@ lsb_16 (guchar *src)
 static gboolean grow_buffer (struct bmp_progressive_state *State,
                              GError **error)
 {
-  guchar *tmp = g_try_realloc (State->buff, State->BufferSize);
+  guchar *tmp;
+
+  if (State->BufferSize == 0) {
+    g_set_error (error,
+		 GDK_PIXBUF_ERROR,
+		 GDK_PIXBUF_ERROR_CORRUPT_IMAGE,
+		 _("BMP image has bogus header data"));
+    State->read_state = READ_STATE_ERROR;
+    return FALSE;
+  }
+
+  tmp = g_try_realloc (State->buff, State->BufferSize);
+
   if (!tmp) {
     g_set_error (error,
 		 GDK_PIXBUF_ERROR,
@@ -226,6 +240,7 @@ static gboolean grow_buffer (struct bmp_progressive_state *State,
     State->read_state = READ_STATE_ERROR;
     return FALSE;
   }
+
   State->buff = tmp;
   return TRUE;
 }
@@ -295,16 +310,23 @@ static gboolean DecodeHeader(unsigned char *BFH, unsigned char *BIH,
 		State->Header.height = -State->Header.height;
 		State->Header.Negative = 1;
 	}
-	if (State->Header.width < 0) {
-		State->Header.width = -State->Header.width;
-		State->Header.Negative = 0;
+
+	if (State->Header.Negative && 
+	    (State->Compressed != BI_RGB && State->Compressed != BI_BITFIELDS))
+	{
+		g_set_error (error,
+			     GDK_PIXBUF_ERROR,
+			     GDK_PIXBUF_ERROR_CORRUPT_IMAGE,
+			     _("Topdown BMP images cannot be compressed"));
+		State->read_state = READ_STATE_ERROR;
+		return FALSE;
 	}
 
-	if (State->Header.width == 0 || State->Header.height == 0 ||
+	if (State->Header.width <= 0 || State->Header.height == 0 ||
 	    (State->Compressed == BI_RLE4 && State->Type != 4)    ||
 	    (State->Compressed == BI_RLE8 && State->Type != 8)	  ||
 	    (State->Compressed == BI_BITFIELDS && !(State->Type == 16 || State->Type == 32)) ||
-	    State->Compressed > BI_BITFIELDS) {
+	    (State->Compressed > BI_BITFIELDS)) {
 		g_set_error (error,
 			     GDK_PIXBUF_ERROR,
 			     GDK_PIXBUF_ERROR_CORRUPT_IMAGE,
@@ -393,8 +415,18 @@ static gboolean DecodeHeader(unsigned char *BFH, unsigned char *BIH,
 		State->read_state = READ_STATE_PALETTE;
 		State->BufferSize = lsb_32 (&BFH[10]) - 14 - State->Header.size; 
 	} else if (State->Compressed == BI_RGB) {
-		State->read_state = READ_STATE_DATA;
-		State->BufferSize = State->LineWidth;
+		if (State->BufferSize < lsb_32 (&BFH[10]))
+		{
+			/* skip over padding between headers and image data */
+			State->read_state = READ_STATE_HEADERS;
+			State->BufferDone = State->BufferSize;
+			State->BufferSize = lsb_32 (&BFH[10]);
+		}
+		else
+		{
+			State->read_state = READ_STATE_DATA;
+			State->BufferSize = State->LineWidth;
+		}
 	} else if (State->Compressed == BI_BITFIELDS) {
 		State->read_state = READ_STATE_BITMASKS;
 		State->BufferSize = 12;
@@ -430,7 +462,7 @@ static gboolean DecodeColormap (guchar *buff,
 		return TRUE;
 	}
 
-	State->Colormap = g_malloc ((1 << State->Header.depth) * sizeof (*State->Colormap));
+	State->Colormap = g_malloc0 ((1 << State->Header.depth) * sizeof (*State->Colormap));
 	for (i = 0; i < State->Header.n_colors; i++)
 
 	{
@@ -1008,12 +1040,15 @@ DoCompressed(struct bmp_progressive_state *context, GError **error)
 	}
 	if (context->updated_func != NULL) {
 		if (context->compr.y > y)
+		{
+			gint new_y = MIN (context->compr.y, context->Header.height);
 			(*context->updated_func) (context->pixbuf,
 						  0,
-						  y,
+						  context->Header.height - new_y,
 						  context->Header.width,
-						  context->compr.y - y,
+						  new_y - y,
 						  context->user_data);
+		}
 
 	}
 
@@ -1039,10 +1074,14 @@ gdk_pixbuf__bmp_image_load_increment(gpointer data,
 
 	gint BytesToCopy;
 
+	g_print ("load_inc\n");
 	if (context->read_state == READ_STATE_DONE)
 		return TRUE;
 	else if (context->read_state == READ_STATE_ERROR)
+	{
+		g_print ("ah\n");
 		return FALSE;
+	}
 
 	while (size > 0) {
 		if (context->BufferDone < context->BufferSize) {	/* We still
@@ -1068,25 +1107,37 @@ gdk_pixbuf__bmp_image_load_increment(gpointer data,
 			if (!DecodeHeader (context->buff,
 					   context->buff + 14, context,
 					   error))
+			{
+				g_print ("bla\n");
 				return FALSE;
+			}
 
 			break;
 
 		case READ_STATE_PALETTE:
 			if (!DecodeColormap (context->buff, context, error))
+			{
+				g_print ("bleh\n");
 				return FALSE;
+			}
 			break;
 
 		case READ_STATE_BITMASKS:
 			if (!decode_bitmasks (context->buff, context, error))
+			{
+				g_print ("blurb\n");
 				return FALSE;
+			}
 			break;
 
 		case READ_STATE_DATA:
 			if (context->Compressed == BI_RGB || context->Compressed == BI_BITFIELDS)
 				OneLine (context);
 			else if (!DoCompressed (context, error))
+			{
+				g_print ("blurb\n");
 				return FALSE;
+			}
 
 			break;
 		case READ_STATE_DONE:
@@ -1101,12 +1152,133 @@ gdk_pixbuf__bmp_image_load_increment(gpointer data,
 	return TRUE;
 }
 
+/* for our convenience when filling file header */
+#define put16(buf,data)	{ guint16 x; \
+			  x = GUINT16_TO_LE (data); \
+			  memcpy(buf, &x, 2); \
+			  buf += 2; }
+#define put32(buf,data)	{ guint32 x; \
+			  x = GUINT32_TO_LE (data); \
+			  memcpy(buf, &x, 4); \
+			  buf += 4; }
+
+static gboolean
+gdk_pixbuf__bmp_image_save_to_callback (GdkPixbufSaveFunc   save_func,
+					gpointer            user_data,
+					GdkPixbuf          *pixbuf, 
+					gchar             **keys,
+					gchar             **values,
+					GError            **error)
+{
+	guint width, height, channel, size, stride, src_stride, x, y;
+	guchar BFH_BIH[54], *pixels, *buf, *src, *dst, *dst_line;
+	gboolean ret;
+
+	width = gdk_pixbuf_get_width (pixbuf);
+	height = gdk_pixbuf_get_height (pixbuf);
+	channel = gdk_pixbuf_get_n_channels (pixbuf);
+	pixels = gdk_pixbuf_get_pixels (pixbuf);
+	src_stride = gdk_pixbuf_get_rowstride (pixbuf);
+	stride = (width * 3 + 3) & ~3;
+	size = stride * height;
+
+	/* filling BFH */
+	dst = BFH_BIH;
+	*dst++ = 'B';			/* bfType */
+	*dst++ = 'M';
+	put32 (dst, size + 14 + 40);	/* bfSize */
+	put32 (dst, 0);			/* bfReserved1 + bfReserved2 */
+	put32 (dst, 14 + 40);		/* bfOffBits */
+
+	/* filling BIH */
+	put32 (dst, 40);		/* biSize */
+	put32 (dst, width);		/* biWidth */
+	put32 (dst, height);		/* biHeight */
+	put16 (dst, 1);			/* biPlanes */
+	put16 (dst, 24);		/* biBitCount */
+	put32 (dst, BI_RGB);		/* biCompression */
+	put32 (dst, size);		/* biSizeImage */
+	put32 (dst, 0);			/* biXPelsPerMeter */
+	put32 (dst, 0);			/* biYPelsPerMeter */
+	put32 (dst, 0);			/* biClrUsed */
+	put32 (dst, 0);			/* biClrImportant */
+
+	if (!save_func (BFH_BIH, 14 + 40, error, user_data))
+		return FALSE;
+
+	dst_line = buf = g_try_malloc (size);
+	if (!buf) {
+		g_set_error (error,
+			     GDK_PIXBUF_ERROR,
+			     GDK_PIXBUF_ERROR_INSUFFICIENT_MEMORY,
+			     _("Couldn't allocate memory for saving BMP file"));
+		return FALSE;
+	}
+
+	/* saving as a bottom-up bmp */
+	pixels += (height - 1) * src_stride;
+	for (y = 0; y < height; ++y, pixels -= src_stride, dst_line += stride) {
+		dst = dst_line;
+		src = pixels;
+		for (x = 0; x < width; ++x, dst += 3, src += channel) {
+			dst[0] = src[2];
+			dst[1] = src[1];
+			dst[2] = src[0];
+		}
+	}
+	ret = save_func (buf, size, error, user_data);
+	g_free (buf);
+
+	return ret;
+}
+
+static gboolean
+save_to_file_cb (const gchar *buf,
+		 gsize count,
+		 GError **error,
+		 gpointer data)
+{
+	gint bytes;
+	
+	while (count > 0) {
+		bytes = fwrite (buf, sizeof (gchar), count, (FILE *) data);
+		if (bytes <= 0)
+			break;
+		count -= bytes;
+		buf += bytes;
+	}
+
+	if (count) {
+		g_set_error (error,
+			     GDK_PIXBUF_ERROR,
+			     GDK_PIXBUF_ERROR_FAILED,
+			     _("Couldn't write to BMP file"));
+		return FALSE;
+	}
+	
+	return TRUE;
+}
+
+static gboolean
+gdk_pixbuf__bmp_image_save (FILE          *f, 
+                            GdkPixbuf     *pixbuf, 
+                            gchar        **keys,
+                            gchar        **values,
+                            GError       **error)
+{
+	return gdk_pixbuf__bmp_image_save_to_callback (save_to_file_cb,
+						       f, pixbuf, keys,
+						       values, error);
+}
+
 void
 MODULE_ENTRY (bmp, fill_vtable) (GdkPixbufModule *module)
 {
 	module->begin_load = gdk_pixbuf__bmp_image_begin_load;
 	module->stop_load = gdk_pixbuf__bmp_image_stop_load;
 	module->load_increment = gdk_pixbuf__bmp_image_load_increment;
+	module->save = gdk_pixbuf__bmp_image_save;
+	module->save_to_callback = gdk_pixbuf__bmp_image_save_to_callback;
 }
 
 void
@@ -1132,7 +1304,7 @@ MODULE_ENTRY (bmp, fill_info) (GdkPixbufFormat *info)
 	info->description = N_("The BMP image format");
 	info->mime_types = mime_types;
 	info->extensions = extensions;
-	info->flags = GDK_PIXBUF_FORMAT_THREADSAFE;
+	info->flags = GDK_PIXBUF_FORMAT_WRITABLE | GDK_PIXBUF_FORMAT_THREADSAFE;
 	info->license = "LGPL";
 }
 
