@@ -18,7 +18,6 @@
  */
 
 #include <config.h>
-#include "gtkalias.h"
 #include "gtkcellview.h"
 #include "gtkcelllayout.h"
 #include "gtkintl.h"
@@ -26,6 +25,7 @@
 #include "gtkcellrenderertext.h"
 #include "gtkcellrendererpixbuf.h"
 #include <gobject/gmarshal.h>
+#include "gtkalias.h"
 
 typedef struct _GtkCellViewCellInfo GtkCellViewCellInfo;
 struct _GtkCellViewCellInfo
@@ -76,11 +76,13 @@ static void        gtk_cell_view_size_allocate            (GtkWidget        *wid
                                                            GtkAllocation    *allocation);
 static gboolean    gtk_cell_view_expose                   (GtkWidget        *widget,
                                                            GdkEventExpose   *event);
-static void        gtk_cell_view_set_valuesv              (GtkCellView      *cellview,
-                                                           GtkCellRenderer  *renderer,
-                                                           va_list           args);
+static void        gtk_cell_view_set_value                (GtkCellView     *cell_view,
+                                                           GtkCellRenderer *renderer,
+                                                           gchar           *property,
+                                                           GValue          *value);
 static GtkCellViewCellInfo *gtk_cell_view_get_cell_info   (GtkCellView      *cellview,
                                                            GtkCellRenderer  *renderer);
+static void        gtk_cell_view_set_cell_data            (GtkCellView      *cell_view);
 
 
 static void        gtk_cell_view_cell_layout_pack_start        (GtkCellLayout         *layout,
@@ -259,7 +261,7 @@ gtk_cell_view_set_property (GObject      *object,
           else
             g_warning ("Don't know color `%s'", g_value_get_string (value));
 
-          g_object_notify (object, "background_gdk");
+          g_object_notify (object, "background-gdk");
         }
         break;
       case PROP_BACKGROUND_GDK:
@@ -442,7 +444,7 @@ gtk_cell_view_expose (GtkWidget      *widget,
                           widget->allocation.width,
                           widget->allocation.height);
 
-      g_object_unref (G_OBJECT (gc));
+      g_object_unref (gc);
     }
 
   /* set cell data (if available) */
@@ -534,7 +536,7 @@ gtk_cell_view_get_cell_info (GtkCellView     *cellview,
   return NULL;
 }
 
-void
+static void
 gtk_cell_view_set_cell_data (GtkCellView *cell_view)
 {
   GList *i;
@@ -594,7 +596,7 @@ gtk_cell_view_cell_layout_pack_start (GtkCellLayout   *layout,
   g_return_if_fail (GTK_IS_CELL_RENDERER (renderer));
   g_return_if_fail (!gtk_cell_view_get_cell_info (cellview, renderer));
 
-  g_object_ref (G_OBJECT (renderer));
+  g_object_ref (renderer);
   gtk_object_sink (GTK_OBJECT (renderer));
 
   info = g_new0 (GtkCellViewCellInfo, 1);
@@ -617,7 +619,7 @@ gtk_cell_view_cell_layout_pack_end (GtkCellLayout   *layout,
   g_return_if_fail (GTK_IS_CELL_RENDERER (renderer));
   g_return_if_fail (!gtk_cell_view_get_cell_info (cellview, renderer));
 
-  g_object_ref (G_OBJECT (renderer));
+  g_object_ref (renderer);
   gtk_object_sink (GTK_OBJECT (renderer));
 
   info = g_new0 (GtkCellViewCellInfo, 1);
@@ -659,7 +661,7 @@ gtk_cell_view_cell_layout_clear (GtkCellLayout *layout)
       GtkCellViewCellInfo *info = (GtkCellViewCellInfo *)cellview->priv->cell_list->data;
 
       gtk_cell_view_cell_layout_clear_attributes (layout, info->cell);
-      g_object_unref (G_OBJECT (info->cell));
+      g_object_unref (info->cell);
       g_free (info);
       cellview->priv->cell_list = g_list_delete_link (cellview->priv->cell_list, 
 						      cellview->priv->cell_list);
@@ -763,7 +765,7 @@ gtk_cell_view_new (void)
 {
   GtkCellView *cellview;
 
-  cellview = GTK_CELL_VIEW (g_object_new (gtk_cell_view_get_type (), NULL));
+  cellview = g_object_new (gtk_cell_view_get_type (), NULL);
 
   return GTK_WIDGET (cellview);
 }
@@ -794,7 +796,7 @@ gtk_cell_view_new_with_text (const gchar *text)
 
   g_value_init (&value, G_TYPE_STRING);
   g_value_set_string (&value, text);
-  gtk_cell_view_set_values (cellview, renderer, "text", &value, NULL);
+  gtk_cell_view_set_value (cellview, renderer, "text", &value);
   g_value_unset (&value);
 
   return GTK_WIDGET (cellview);
@@ -828,7 +830,7 @@ gtk_cell_view_new_with_markup (const gchar *markup)
 
   g_value_init (&value, G_TYPE_STRING);
   g_value_set_string (&value, markup);
-  gtk_cell_view_set_values (cellview, renderer, "markup", &value, NULL);
+  gtk_cell_view_set_value (cellview, renderer, "markup", &value);
   g_value_unset (&value);
 
   return GTK_WIDGET (cellview);
@@ -860,7 +862,7 @@ gtk_cell_view_new_with_pixbuf (GdkPixbuf *pixbuf)
 
   g_value_init (&value, GDK_TYPE_PIXBUF);
   g_value_set_object (&value, pixbuf);
-  gtk_cell_view_set_values (cellview, renderer, "pixbuf", &value, NULL);
+  gtk_cell_view_set_value (cellview, renderer, "pixbuf", &value);
   g_value_unset (&value);
 
   return GTK_WIDGET (cellview);
@@ -878,7 +880,7 @@ gtk_cell_view_new_with_pixbuf (GdkPixbuf *pixbuf)
  *
  * Since: 2.6
  */
-void
+static void
 gtk_cell_view_set_value (GtkCellView     *cell_view,
                          GtkCellRenderer *renderer,
                          gchar           *property,
@@ -894,52 +896,6 @@ gtk_cell_view_set_value (GtkCellView     *cell_view,
   gtk_widget_queue_draw (GTK_WIDGET (cell_view));
 }
 
-static void
-gtk_cell_view_set_valuesv (GtkCellView     *cell_view,
-                           GtkCellRenderer *renderer,
-                           va_list          args)
-{
-  gchar *attribute;
-  GValue *value;
-
-  attribute = va_arg (args, gchar *);
-
-  while (attribute)
-    {
-      value = va_arg (args, GValue *);
-      gtk_cell_view_set_value (cell_view, renderer, attribute, value);
-      attribute = va_arg (args, gchar *);
-    }
-}
-
-/**
- * gtk_cell_view_set_values:
- * @cell_view: a #GtkCellView widget
- * @renderer: one of the renderers of @cell_view
- * @Varargs: a list of pairs of property names and #GValue<!-- -->s,
- *   finished by %NULL
- * 
- * Sets multiple properties of a cell renderer of @cell_view, and
- * makes sure the display of @cell_view is updated.
- *
- * Since: 2.6
- */
-void
-gtk_cell_view_set_values (GtkCellView     *cell_view,
-                          GtkCellRenderer *renderer,
-                          ...)
-{
-  va_list args;
-
-  g_return_if_fail (GTK_IS_CELL_VIEW (cell_view));
-  g_return_if_fail (GTK_IS_CELL_RENDERER (renderer));
-  g_return_if_fail (gtk_cell_view_get_cell_info (cell_view, renderer));
-
-  va_start (args, renderer);
-  gtk_cell_view_set_valuesv (cell_view, renderer, args);
-  va_end (args);
-}
-
 /**
  * gtk_cell_view_set_model:
  * @cell_view: a #GtkCellView
@@ -964,14 +920,14 @@ gtk_cell_view_set_model (GtkCellView  *cell_view,
         gtk_tree_row_reference_free (cell_view->priv->displayed_row);
       cell_view->priv->displayed_row = NULL;
 
-      g_object_unref (G_OBJECT (cell_view->priv->model));
+      g_object_unref (cell_view->priv->model);
       cell_view->priv->model = NULL;
     }
 
   cell_view->priv->model = model;
 
   if (cell_view->priv->model)
-    g_object_ref (G_OBJECT (cell_view->priv->model));
+    g_object_ref (cell_view->priv->model);
 }
 
 /**
@@ -1011,6 +967,18 @@ gtk_cell_view_set_displayed_row (GtkCellView *cell_view,
   gtk_widget_queue_draw (GTK_WIDGET (cell_view));
 }
 
+/**
+ * gtk_cell_view_get_displayed_row:
+ * @cell_view: a #GtkCellView
+ *
+ * Returns a #GtkTreePath referring to the currently 
+ * displayed row. If no row is currently displayed, 
+ * %NULL is returned.
+ *
+ * Returns: the currently displayed row or %NULL
+ *
+ * Since: 2.6
+ */
 GtkTreePath *
 gtk_cell_view_get_displayed_row (GtkCellView *cell_view)
 {
@@ -1082,7 +1050,7 @@ gtk_cell_view_set_background_color (GtkCellView    *cell_view,
       if (!cell_view->priv->background_set)
         {
           cell_view->priv->background_set = TRUE;
-          g_object_notify (G_OBJECT (cell_view), "background_set");
+          g_object_notify (G_OBJECT (cell_view), "background-set");
         }
 
       cell_view->priv->background = *color;
@@ -1092,9 +1060,11 @@ gtk_cell_view_set_background_color (GtkCellView    *cell_view,
       if (cell_view->priv->background_set)
         {
           cell_view->priv->background_set = FALSE;
-          g_object_notify (G_OBJECT (cell_view), "background_set");
+          g_object_notify (G_OBJECT (cell_view), "background-set");
         }
     }
+
+  gtk_widget_queue_draw (GTK_WIDGET (cell_view));
 }
 
 /**
@@ -1116,6 +1086,8 @@ gtk_cell_view_get_cell_renderers (GtkCellView *cell_view)
 
   g_return_val_if_fail (cell_view != NULL, NULL);
 
+  gtk_cell_view_set_cell_data (cell_view);
+
   for (list = cell_view->priv->cell_list; list; list = list->next)
     {
       GtkCellViewCellInfo *info = (GtkCellViewCellInfo *)list->data;
@@ -1125,3 +1097,6 @@ gtk_cell_view_get_cell_renderers (GtkCellView *cell_view)
 
   return g_list_reverse (retval);
 }
+
+#define __GTK_CELL_VIEW_C__
+#include "gtkaliasdef.c"
