@@ -30,6 +30,7 @@
 #include <stdlib.h>
 #include "gtkhscale.h"
 #include "gtkintl.h"
+#include "gtkalias.h"
 
 static gpointer parent_class;
 
@@ -257,3 +258,6 @@ gtk_hscale_get_layout_offsets (GtkScale *scale,
   *x += widget->allocation.x;
   *y += widget->allocation.y;
 }
+
+#define __GTK_HSCALE_C__
+#include "gtkaliasdef.c"
