@@ -23,11 +23,11 @@
 #include <glib.h>
 #include <glib/gprintf.h>
 #include <gobject/gvaluecollector.h>
-#include "gtkalias.h"
 #include "gtktreemodel.h"
 #include "gtktreeview.h"
 #include "gtktreeprivate.h"
 #include "gtkmarshalers.h"
+#include "gtkalias.h"
 
 
 #define INITIALIZE_TREE_ITER(Iter) \
@@ -734,6 +734,9 @@ gtk_tree_path_prev (GtkTreePath *path)
 {
   g_return_val_if_fail (path != NULL, FALSE);
 
+  if (path->depth == 0)
+    return FALSE;
+
   if (path->indices[path->depth - 1] == 0)
     return FALSE;
 
@@ -2019,3 +2022,6 @@ gtk_tree_row_reference_reordered (GObject     *proxy,
 
   gtk_tree_row_ref_reordered ((RowRefList *)g_object_get_data (proxy, ROW_REF_DATA_STRING), path, iter, new_order);
 }
+
+#define __GTK_TREE_MODEL_C__
+#include "gtkaliasdef.c"
